# Copyright 2013 Openstack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import netaddr
import webob.dec
import webob.exc

from wafflehaus.base import WafflehausBase
import wafflehaus.resource_filter as rf


class DefaultIPPolicy(WafflehausBase):

    def __init__(self, app, conf):
        super(DefaultIPPolicy, self).__init__(app, conf)
        self.log.name = conf.get('log_name', __name__)
        self.log.info('Starting wafflehaus default ip policy middleware')
        self.resource = conf.get('resource', 'POST /v2.0/subnets')
        self.resources = rf.parse_resources(self.resource)

    def _pools_from_ipset(self, ipset):
        cidrs = ipset.iter_cidrs()
        if len(cidrs) == 0:
            return []
        if len(cidrs) == 1:
            return [dict(start=str(cidrs[0][0]),
                         end=str(cidrs[0][-1]))]

        pool_start = cidrs[0][0]
        prev_cidr_end = cidrs[0][-1]
        pools = []
        for cidr in cidrs[1:]:
            cidr_start = cidr[0]
            if prev_cidr_end + 1 != cidr_start:
                pools.append(dict(start=str(pool_start),
                                  end=str(prev_cidr_end)))
                pool_start = cidr_start
            prev_cidr_end = cidr[-1]
        pools.append(dict(start=str(pool_start), end=str(prev_cidr_end)))
        return pools

    def _get_default_allocation_pools(self, subnet):
        alloc_pools = {}
        cidr_net = netaddr.IPNetwork(subnet["cidr"])
        starting_index = 5 if subnet.get("ip_version") == 4 else 10
        start = cidr_net[starting_index]
        end = cidr_net[-2]
        alloc_pools = [{"start": str(start),
                       "end": str(end)}]
        return alloc_pools

    def _modify_allocation_pools(self, subnet):
        alloc_pools = subnet.get('allocation_pools')
        default_alloc_pools = self._get_default_allocation_pools(subnet)
        default_start = netaddr.IPAddress(default_alloc_pools[0]["start"])
        default_end = netaddr.IPAddress(default_alloc_pools[0]["end"])
        default_set = netaddr.IPSet(netaddr.IPRange(default_start,
                                                    default_end).cidrs())
        final_set = netaddr.IPSet()
        for p in alloc_pools:
            start = netaddr.IPAddress(p["start"])
            end = netaddr.IPAddress(p["end"])
            alloc_pool_ip_set = netaddr.IPSet(
                netaddr.IPRange(
                    netaddr.IPAddress(start), netaddr.IPAddress(end)).cidrs())
            final_set.update(default_set & alloc_pool_ip_set)
        alloc_pools = self._pools_from_ipset(final_set)
        return alloc_pools

    def _filter_policy(self, req):
        body = req.body
        try:
            body_json = json.loads(body)
        except ValueError:
            return webob.exc.HTTPBadRequest
        subnets = body_json.get('subnets')
        subnet = body_json.get('subnet')
        body_json["subnets"] = []
        if subnets is None and subnet is None:
            """If this is true there is nothing to work with let app error."""
            return self.app
        single = False
        if subnets is None:
            """If this is true then it's a single, put it in list."""
            single = True
            subnets = [subnet]
        for subnet in subnets:
            alloc_pools = subnet.get('allocation_pools')
            if alloc_pools is None:
                alloc_pools = self._get_default_allocation_pools(subnet)
            else:
                alloc_pools = self._modify_allocation_pools(subnet)
            subnet["allocation_pools"] = alloc_pools
            body_json["subnets"].append(subnet)
        if single:
            body_json["subnet"] = body_json.pop("subnets")[0]
        req.body = json.dumps(body_json)
        self.body = req.body
        return self.app

    @webob.dec.wsgify
    def __call__(self, req):
        if not self.enabled:
            return self.app

        if not rf.matched_request(req, self.resources):
            return self.app
        return self._filter_policy(req)


def filter_factory(global_conf, **local_conf):
    """Returns a WSGI filter app for use with paste.deploy."""
    conf = global_conf.copy()
    conf.update(local_conf)

    def block_resource(app):
        return DefaultIPPolicy(app, conf)
    return block_resource



# Copyright 2013 Openstack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import netaddr
import webob.dec
import webob.exc

from wafflehaus.base import WafflehausBase


class LastIpCheck(WafflehausBase):
    def __init__(self, app, conf):
        super(LastIpCheck, self).__init__(app, conf)
        self.log.name = conf.get('log_name', __name__)
        self.log.info('Starting wafflehaus last_ip_check middleware')

    def _check_basics(self, req):
        if req.content_length == 0:
            return False
        method = req.method
        if method not in ['PUT']:
            return False
        url = req.path
        url_parts = url.split('/')
        if 'ports' not in url and 'ports' not in url_parts[len(url_parts) - 2]:
            return False
        return True

    def _should_run(self, req):
        basic_check = self._check_basics(req)
        if isinstance(basic_check, webob.exc.HTTPException) or not basic_check:
            return basic_check
        body = req.body
        try:
            body_json = json.loads(body)
        except ValueError:
            return webob.exc.HTTPBadRequest
        port_info = body_json.get('port')
        if port_info is None:
            return False
        fixed_ips = port_info.get('fixed_ips')
        if fixed_ips is None:
            return False
        self.fixed_ips = fixed_ips
        return True

    def _is_last_ip(self, req):
        if not hasattr(self, 'fixed_ips') or self.fixed_ips is None:
            return self.app
        if len(self.fixed_ips) == 0:
            return webob.exc.HTTPForbidden()
        for fixed_ip in self.fixed_ips:
            ip_str = fixed_ip.get('ip_address')
            ip = netaddr.IPAddress(ip_str)
            if ip.version == 4:
                """If there is an ipv4 there, we're good"""
                return self.app
        self.log.error('Attempting to remove last ipv4')
        return webob.exc.HTTPForbidden()

    @webob.dec.wsgify
    def __call__(self, req):
        if not self.enabled:
            return self.app

        res = self._should_run(req)
        if isinstance(res, webob.exc.HTTPException):
            return res
        if not res:
            return self.app
        return self._is_last_ip(req)


def filter_factory(global_conf, **local_conf):
    """Returns a WSGI filter app for use with paste.deploy."""
    conf = global_conf.copy()
    conf.update(local_conf)

    def check_last_ip(app):
        return LastIpCheck(app, conf)
    return check_last_ip



# Copyright 2013 Openstack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron import context

from wafflehaus.try_context.context_filter import BaseContextStrategy


class NeutronContextFilter(BaseContextStrategy):
    def __init__(self, key):
        super(NeutronContextFilter, self).__init__(key)
        self.neutron_ctx = context

    def _process_roles(self, roles):
        if not self.context.roles:
            self.context.roles = []
        if roles is None:
            return
        roles = [r.strip() for r in roles.split(',')]
        for role in roles:
            if role not in self.context.roles:
                self.context.roles.append(role)

    def load_context(self, req):
        super(NeutronContextFilter, self).load_context(req)
        ctx = self.neutron_ctx.get_admin_context()
        self.context = ctx
        self._process_roles(req.headers.get('X_ROLES', ''))
        req.environ['neutron.context'] = self.context



#!/usr/bin/python
# -*- encoding: utf-8 -*-
# Copyright 2013 Openstack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import setuptools

version = '0.1.1'

setuptools.setup(
    name='wafflehaus.neutron',
    version=version,
    description="A collection of middleware for openstack neutron",
    classifiers=[
      "Development Status :: 5 - Production/Stable",
      "Environment :: Web Environment",
      "Framework :: Paste",
      "Intended Audience :: Developers",
      "Intended Audience :: System Administrators",
      "License :: OSI Approved :: Apache Software License",
      "Operating System :: OS Independent",
      "Programming Language :: Python :: 2.7",
      "Programming Language :: Python :: 2.6",
      "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware"
    ], 
    keywords=['openstack'],
    author='Justin Hammond',
    author_email='justin.hammond@rackspace.com',
    url='https://github.com/roaet/wafflehaus.neutron',
    license='Apache Software License',
    packages=setuptools.find_packages(exclude=['tests']),
    long_description=open('README.md').read(),
    zip_safe=False,
    install_requires=[
      "webob",
      "dnspython",
    ],
    namespace_packages=['wafflehaus'],
)



# Copyright 2013 Openstack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import webob.dec

from wafflehaus.base import WafflehausBase
import wafflehaus.resource_filter as rf


class TrustedSharedNetwork(WafflehausBase):

    def __init__(self, app, conf):
        super(TrustedSharedNetwork, self).__init__(app, conf)
        self.log.name = conf.get('log_name', __name__)
        self.log.info('Starting wafflehaus trusted shared nets middleware')
        self.resource = conf.get('resource', 'GET /v2.0/networks{.format}')
        self.resources = rf.parse_resources(self.resource)

        self.trusted_nets = conf.get('trusted', '')
        if isinstance(self.trusted_nets, basestring):
            self.trusted_nets = self.trusted_nets.split()
        self.trusted_nets = set(self.trusted_nets)

    def _shared_nets_filter(self, req):
        if "shared" not in req.GET:
            return self.app
        return self._sanitize_shared_nets(req)

    def _sanitize_shared_nets(self, req):
        headers = req.headers
        response = req.get_response(self.app)
        body = response.json
        networks = body.get('networks')

        whitelist = set(headers.get('X_NETWORK_WHITELIST', '').split(','))
        blacklist = set(headers.get('X_NETWORK_BLACKLIST', '').split(','))

        # Collect the shared network ids
        shared_nets = set(n['id'] for n in networks if n['shared'])

        # Collect the unshared network ids
        unshared_nets = set(n['id'] for n in networks if not n['shared'])

        # Only allow configured or whitelisted shared networks
        # But definitely remove blacklisted networks
        okay_nets = set(n for n in shared_nets if n in
                        whitelist.union(self.trusted_nets) - blacklist)

        # Use the networks that are either ok or unshared
        body['networks'] = [n for n in networks if n['id'] in
                            okay_nets.union(unshared_nets)]
        response.json = body

        return response

    @webob.dec.wsgify
    def __call__(self, req):
        if not self.enabled:
            return self.app
        if self.testing or not rf.matched_request(req, self.resources):
            return self.app
        return self._shared_nets_filter(req)


def filter_factory(global_conf, **local_conf):
    """Returns a WSGI filter app for use with paste.deploy."""
    conf = global_conf.copy()
    conf.update(local_conf)

    def trusted_shared_nets(app):
        return TrustedSharedNetwork(app, conf)
    return trusted_shared_nets
